"""
Module provides access to logger.

This needs to be used sparingly, prefer to raise specific exceptions instead.

Import functions:
    - change_log_level
"""
import os
import sys

import toml

from loguru import logger


LOG_LEVELS = ['CRITICAL', 'ERROR', 'WARNING', 'SUCCESS', 'INFO', 'DEBUG', 'TRACE']


def get_version():
    project_toml = os.path.abspath(os.path.join(os.path.dirname( __file__ ), '..', 'pyproject.toml'))
    with open(project_toml, 'r') as handle:
        project_metadata = toml.load(handle)
    return project_metadata["tool"]["poetry"]["version"]


def configure_logger(level: int) -> None:
    """
    Configure the logger.
    """
    size = len(LOG_LEVELS)
    if level >= size:
        level = size - 1
    log_level = LOG_LEVELS[level]

    config = {
        "handlers": [
            {
                "sink": sys.stdout,
                "format": """
    -------------------------------------------------------
    <level>{level}</level>
    -------
    TIME: <green>{time}</green>
    FILE: {name}:L{line} <blue>{function}(...)</blue>
    <level>{message}</level>
    -------------------------------------------------------
    """,
                "colorize": True,
                "level": log_level,
            },
            {
                "sink": "file.log",
                "rotation": "500MB",
                "retention": "10 days",
                "format": "{time} {level} -\n{message}\n--------------------\n",
                "level": log_level,
            },
        ],
    }
    logger.configure(**config)
    logger.enable(__name__)
    



from urllib.parse import urljoin

import requests

from skit_auth import constants as const
from loguru import logger


def check_network_error(response: requests.Response) -> None:
    """
    Raise a network error if the response is not successful.

    :param response: The response from the API Gateway.
    :type response: requests.Response
    :return: None
    :rtype: None
    """
    if response.status_code != const.HTTP_SUCCESS:
        raise ValueError(
            f"{response.status_code}: Unable to get "
            f"token from API Gateway, error: {response.text}."
        )


def get_default_token(url: str, email: str, password: str) -> str:
    """
    Get a token from the API Gateway.

    :param url: The URL of the API Gateway.
    :type url: str
    :param email: The email address of the user or IAM.
    :type email: str
    :param password: The password of the user or IAM.
    :type password: str
    :return: JSON Web Token
    :rtype: str
    """
    payload = {const.EMAIL: email, const.PASSWORD: password}
    response = requests.post(urljoin(url, const.ROUTE_OAUTH), json=payload)
    check_network_error(response)
    return response.json().get(const.ACCESS_TOKEN)


def get_org_token(url: str, email: str, password: str, org_id: int) -> str:
    """
    Get a token from the API Gateway for an organization with id = `org_id`.

    :param url: The URL of the API Gateway.
    :type url: str
    :param org_id: The ID of the organization.
    :type org_id: int
    :param token: The token provided for skit's organization.
    :type token: str
    :return: JSON Web Token
    :rtype: str
    """

    payload = {const.ORG_ID: org_id}
    token = get_default_token(url, email, password)
    headers = {const.AUTHORIZATION: f"Bearer {token}"}
    response = requests.post(
        urljoin(url, const.ROUTE_CHANGE_ORG), json=payload, headers=headers
    )
    check_network_error(response)
    logger.debug(f"Acquired organization token for {org_id=}.")
    logger.debug(f"{response.json()}")
    return response.json().get(const.ACCESS_TOKEN)



import os
import argparse
import getpass

import toml

from skit_auth import auth, __version__
from skit_auth import constants as const
from skit_auth import utils

VERSION = __version__


class Password(argparse.Action):
    def __call__(self, parser, namespace, values, option_string):
        values = os.environ.get("SKIT_API_GATEWAY_PASSWORD") or getpass.getpass()
        setattr(namespace, self.dest, values)


def build_cli():
    parser = argparse.ArgumentParser(description=f"skit-auth {VERSION}.\n\nskit.ai's authorization library.")
    parser.add_argument("-v", "--verbose", action="count", help="Increase verbosity.", default=0)
    parser.add_argument(
        "--url",
        type=str,
        help=f"The URL of the API Gateway. [Default={const.DEFAULT_API_GATEWAY_URL}]",
        default=const.DEFAULT_API_GATEWAY_URL,
    )
    parser.add_argument(
        "--email", type=str, required=True, help="The email address of the user or IAM."
    )
    parser.add_argument(
        "-p", "--password", type=str, help="The password of the user or IAM."
        " We can also read it from the SKIT_API_GATEWAY_PASSWORD environment variable."
        " *DON'T* provide a value e.g. -p <password> or --password <password> will fail.",
        action=Password,
        dest="password", nargs=0, default=os.environ.get("SKIT_API_GATEWAY_PASSWORD")
    )
    parser.add_argument(
        "--org-id", type=int, required=False, help="The ID of the organization."
    )
    return parser


def main() -> None:
    """
    Main entry point for the CLI.

    We try to read token from the pipes.
    """
    cli = build_cli()
    args = cli.parse_args()
    utils.configure_logger(args.verbose)

    if not isinstance(args.password, str) or args.password == "":
        raise argparse.ArgumentTypeError("You must provide a password. Use -p or --password.")
    if args.org_id:
        print(auth.get_org_token(args.url, args.email, args.password, args.org_id))
    else:
        print(auth.get_default_token(args.url, args.email, args.password))
